%% Function for assessing fatigue damage of one cross section
% Input: (loading seqence of eight spots [MPa], design life [years], ...
%           ... analysis time [s])
% Output: fail = true or false
% Written by Johan H. Martens, spring 2014

function [fail] = fatiguefail(lseq,yr,eff_t)
cycle=0;    % variable to store stress cycles

%% Extracting stress cycles from load sequence
% for all eigth stress spots around the circumference of the cross section
for s=1:8   
    clear extrema
    % Extracting local extrema (peaks and valleys) of timeseries
    [pks,pkspos]=findpeaks(lseq(:,s));      % finding position and value of peaks
    [vlys,vlyspos]=findpeaks(-lseq(:,s));   % finding position and value of valleys
    vlys=-vlys;
    extrema = [pkspos pks; vlyspos vlys];   % combining peaks and valleys
    extrema = sortrows(extrema,1);          % putting in correct order
    extrema = extrema(:,2);                 % storing values
    extrema(2:length(extrema)+1) = extrema;
    extrema(1) = lseq(1,s);                 % adding first datapoint
    extrema(end+1) = lseq(end,s);           % adding last datapoint
    i=1;                                    % counters        
    k=1;
    res=0;
    while res==0 && length(extrema)>3
        Nr=length(extrema)-3;
        while (i <= Nr)
        % calculate delta amplitudes
        clear dS
        dS(1) = abs(extrema(i+1) - extrema(i  ));
        dS(2) = abs(extrema(i+2) - extrema(i+1));
        dS(3) = abs(extrema(i+3) - extrema(i+2));
            if ((dS(2) <= dS(1)) && (dS(2) <= dS(3)))  % check delta amplitudes
                cycle(k,s) = dS(2);     % storage of the extracted cycle
                k = k + 1;                   
                extrema(i+1:i+2) = [];  % discard points that make out cycle
                res = 0;                % check from beginning for dataset
                i = 1;             
                break
            % if no cycle was extracted, continue to next set of dS's
            else                             
                i = i + 1;
                res = 1;
            end
        end
    end
    % adding residue to itself to extract cycles from residue 
    residue=[extrema; extrema]; 
    i=1;
    res=0;
    while res==0 && length(residue)>3
        Nr=length(residue)-3;
        while (i <= Nr)
        % calculate delta amplitudes
        clear dS
        dS(1) = abs(residue(i+1) - residue(i  ));
        dS(2) = abs(residue(i+2) - residue(i+1));
        dS(3) = abs(residue(i+3) - residue(i+2));   
            if ((dS(2) <= dS(1)) && (dS(2) <= dS(3))) % check delta amplitudes
                cycle(k,s) = dS(2);     % storage of the extracted cycle
                k = k + 1;              
                residue(i+1:i+2) = [];  % discard points that make out cycle
                res = 0;                % check from beginning for dataset
                i = 1;
                break
            % if no cycle was extracted, continue to next set of dS's
            else
                i = i + 1;
                res = 1;
            end
        end
    end
end

%% Finding S-N parameters
% number of times eff_t (effective analysis time) occur during design lifetime
n=yr*365*24*60*60/eff_t;    
% S-N curve data for tubular joints in seawater with cathodic protection (DNV RP)
if n <= 1E6                
    m=3;
    loga=11.764;
elseif n > 1E6
    m=5;
    loga=15.606;
end

%% Accumulating damage in each stress spot
spotD=zeros(1,8);       % vector to store accumulated spot damage
for s=1:8
    for i=1:length(cycle(:,s))  % for all cycles
        if cycle(i,s) > 1       % omit contribution from stress ranges below 1 MPa
            N1=10^(loga-m*log10(cycle(i,s)));
            spotD(1,s)=spotD(1,s) + n/N1;
        end
    end
end

%% Checking for failure and creating output variable
if any(spotD(1,:)>=1)   % if any spot had accumulated damage >= 1
    fail=1;             % fatigue failure of cross section
else
    fail=0;             % no fatigue failure of cross section
end
end % end of function

