%% Function for writing .ftl file of jacket and defining symmetrical members
% Input: (nodes along width of jacket, height/width ratio)
% Output: [Beam definitions, Node definitions, Symmetry definitions]
% Written by Johan H. Martens, spring 2014

function [B,N,S] = jacket_ftl_creator(n,lvls)
a = 2*4.016/(n-1);      % reference distance between nodes
S = cell(5,1);          % preallocating symmetry matrix
TwoDpos=cell(4,1);
N=zeros(4*n^2-4*n,4);   % Preparing nodal position vector
k=(n-1)*a;              % jacket width
i=1;                    % counter for total number of nodes
i1=100;                 % counters for node id (4 faces)
i2=200;
i3=300;
i4=400;
% Finding number of nodes through the height of the jacket (hn)
if lvls==1
    hn=n;
else
    hn=n+(lvls-1)*(n-1);
end

%% Defining nodal positions (face 1 and 4 "owns" edgenodes)
for s=[1 4 2 3] % counter over faces of cube
    switch s
    case 1
        TwoDl=1;
        pos = [0 0 0];  % initial position
        for h=1:hn      % height counter
            for b=1:n   % width counter
                N(i,1:4)=[i1 pos];                
                pos = pos + [a 0 0];
                TwoDpos{s,1}(TwoDl,1:3)=[i1 b h];
                TwoDl=TwoDl+1;
                i1=i1 + 1;
                i=i + 1;
            end
            pos = [0 0 pos(3)]+[0 0 a];
        end
    case 4
        TwoDl=1;
        pos = [k k 0];
        for h=1:hn
            for b=1:n
                N(i,1:4)=[i4 pos];              
                pos = pos + [-a 0 0];
                TwoDpos{s,1}(TwoDl,1:3)=[i4 b h];
                TwoDl=TwoDl+1;
                i4=i4 + 1;
                i=i + 1;
            end
            pos = [k k pos(3)] + [0 0 a];
        end
    case 2
        TwoDl=1;
        pos = [0 k-a 0];
        for h=1:hn
            for b=2:n-1
                N(i,1:4)=[i2 pos];
                pos = pos + [0 -a 0];
                TwoDpos{s,1}(TwoDl,1:3)=[i2 b h];
                TwoDl=TwoDl+1;
                i2=i2 + 1;
                i=i + 1;
            end
            pos = [0 k-a pos(3)] + [0 0 a];
        end
        h=1;
        for edge=1:hn
            TwoDpos{s,1}(TwoDl  ,1:3)=[TwoDpos{1,1}(1+n*(edge-1),1) n h];
            TwoDpos{s,1}(TwoDl+hn,1:3)=[TwoDpos{4,1}(n*edge,1) 1 h];
            TwoDl=TwoDl+1;
            h=h+1;
        end
    case 3
        TwoDl=1;
        pos = [k a 0];
        for h=1:hn
            for b=2:n-1
                N(i,1:4)=[i3 pos];               
                pos = pos + [0 a 0];
                TwoDpos{s,1}(TwoDl,1:3)=[i3 b h];
                TwoDl=TwoDl+1;
                i3=i3 + 1;
                i=i + 1;
            end
            pos = [k a pos(3)]+[0 0 a];
        end
        h=1;
        for edge=1:hn
            TwoDpos{s,1}(TwoDl  ,1:3)=[TwoDpos{4,1}(1+n*(edge-1),1) n h];
            TwoDpos{s,1}(TwoDl+hn,1:3)=[TwoDpos{1,1}(n*edge,1) 1 h];
            TwoDl=TwoDl+1;
            h=h+1;
        end
    end 
end

%% Extracting faces of cube in different subsets and combining them
m = length(N);              % number of nodes
j=1;                        % counter    
for i=1:m                   % for all nodes
    if N(i,1) < 200         % if id of face 1
        N1(j,1)=N(i);
        j=j+1;
    end
end
C1 = nchoosek(N1(:,1),2);   % all possible combinations between two nodes on face 1
j=1;                        % counter
for i=1:m
    if N(i,1) < 300 && N(i) >= 200  % checking for correct id
        N2(j,1)=N(i);
        j=j+1;
    end
    if N(i,2)==0 && N(i,3)==0       % finding relevant corner nodes
        N2(j,1)=N(i);
        j=j+1;
    end
    if N(i,2)==0 && N(i,3)==k       % finding relevant corner nodes
        N2(j,1)=N(i);
        j=j+1;
    end
end
C2 = nchoosek(N2(:,1),2);
j=1; 
for i=1:m
    if N(i,1) < 400 && N(i) >= 300 
        N3(j,1)=N(i);
        j=j+1;
    end
    if N(i,2)==k && N(i,3)==0
        N3(j,1)=N(i);
        j=j+1;
    end
    if N(i,2)==k && N(i,3)==k
        N3(j,1)=N(i);
        j=j+1;
    end
end
C3 = nchoosek(N3(:,1),2);
j=1;
for i=1:m
    if N(i,1) >= 400
        N4(j,1)=N(i);
        j=j+1;
    end
end
C4 = nchoosek(N4(:,1),2);
C = [C1;C2;C3;C4]; % adding combinations of each face to a single vector

%% Removing duplicate node combinations at corners
c = length(C); % number of combinations
dup=0;         % number of duplicates
for i=1:c
    for j=1:c
        if i~=j && (all(C(i,:) == C(j,:)) || all(C(i,:)==[C(j,2) C(j,1)])) && (all(C(j,:) ~= [0 0]))
            C(i,:)=[];
            C(end+1,:)=[0 0];
            dup=dup+1;
        end
    end
end
C=C(1:end-dup,:);    

%% Removal of beams that are collinear (on top of each other)
c=length(C);                                    % number of beam combinations
for i=1:c                                       % for all beams
    % extracting triad coordinates [x1 y1 z1; x2 y2 z2]
    pos=[N(N==C(i,1),2:4); N(N==C(i,2),2:4)];   
    dx=abs(pos(2,1)-pos(1,1));                  % positive length along axes
    dy=abs(pos(2,2)-pos(1,2));
    dz=abs(pos(2,3)-pos(1,3));
    len=sqrt(dx^2+dy^2+dz^2);                   % length of beam i
    % matrix containing unit vector information [beam# length unitvector]:
    uvec(i,1:5)=[i len dx/len dy/len dz/len];   
end
q=1;
remID=0;
for i=1:c       % double loop to check all members against each other
    for j=1:c
        % if two different members have the same unit vector
        if i~=j && all(uvec(i,3:5)==uvec(j,3:5)) 
            % if one of them is longer than the other & they have at least one node in common, then erase the longer one:           
            if uvec(i,2) > uvec(j,2) && (C(i,1)==C(j,1) || C(i,1)==C(j,2) || C(i,2)==C(j,1) || C(i,2)==C(j,2))
                if ~any(remID==i) && abs( uvec(i,2)-uvec(j,2)) > 0.01
                    remID(q,1)=i;
                    q=q+1;
                end
            elseif uvec(i,2) < uvec(j,2) && (C(i,1)==C(j,1) || C(i,1)==C(j,2) || C(i,2)==C(j,1) || C(i,2)==C(j,2))
                if ~any(remID==j) && abs( uvec(i,2)-uvec(j,2)) > 0.01
                    remID(q,1)=j;
                    q=q+1;
                end
            end
        end
    end
end

%% Defining symmetrical members and output variable S
q=1;
c1=1;
c2=1;
c3=1;
c4=1;
c5=1;
S{5,1}=0;
% subdividing beams into four faces and one corner (S{1:5,1})
for i=1:c
    if ~any(remID==i) % If beam was not removed above -> add to final beam defenitions, B
        B(q,1:2)=[C(i,1:2)];
        % Extracting symmetry data into S
        % S will contain beams on each face in S{1:4,1} and corner beams in S{5,1}
        for s1=1:length(C1)
            % if the node combination of beam q are on side one
            if all(B(q,:)==C1(s1,:)) || all([B(q,2) B(q,1)]==C1(s1,:))
                % if both nodes are not corner nodes -> add to beams of side 1 S{1,1}
                if ~(all(N(N==B(q,1),2:3)==[0 0]) && all(N(N==B(q,2),2:3)==[0 0])) && ~(all(N(N==B(q,1),2:3)==[k 0]) && all(N(N==B(q,2),2:3)==[k 0]))
                    S{1,1}(c1,1:3)=[q C(i,1:2)];
                    c1=c1+1;
                % else if both nodes are on a corner and has not been added
                % to corner beam matrix -> add to S{5,1}
                elseif ~any(S{5,1}(:,1)==q)
                    S{5,1}(c5,1:3)=[q C(i,1:2)];
                    c5=c5+1;
                end
            end
        end % Repeat for face 2:4
        for s2=1:length(C2)
            if all(B(q,:)==C2(s2,:)) || all([B(q,2) B(q,1)]==C2(s2,:))
                if ~(all(N(N==B(q,1),2:3)==[0 0]) && all(N(N==B(q,2),2:3)==[0 0])) && ~(all(N(N==B(q,1),2:3)==[0 k]) && all(N(N==B(q,2),2:3)==[0 k]))
                    S{2,1}(c2,1:3)=[q C(i,1:2)];
                    c2=c2+1;
                elseif ~any(S{5,1}(:,1)==q)
                    S{5,1}(c5,1:3)=[q C(i,1:2)];
                    c5=c5+1;
                end
            end
        end
        for s3=1:length(C3)
            if all(B(q,:)==C3(s3,:)) || all([B(q,2) B(q,1)]==C3(s3,:))
                if ~(all(N(N==B(q,1),2:3)==[k k]) && all(N(N==B(q,2),2:3)==[k k]))&& ~(all(N(N==B(q,1),2:3)==[k 0]) && all(N(N==B(q,2),2:3)==[k 0]))
                    S{3,1}(c3,1:3)=[q C(i,1:2)];
                    c3=c3+1;
                elseif ~any(S{5,1}(:,1)==q)
                    S{5,1}(c5,1:3)=[q C(i,1:2)];
                    c5=c5+1;
                end
            end
        end
        for s4=1:length(C4)
            if all(B(q,:)==C4(s4,:)) || all([B(q,2) B(q,1)]==C3(s4,:))
                if ~(all(N(N==B(q,1),2:3)==[k k]) && all(N(N==B(q,2),2:3)==[k k])) && ~(all(N(N==B(q,1),2:3)==[0 k]) && all(N(N==B(q,2),2:3)==[0 k]))
                    S{4,1}(c4,1:3)=[q C(i,1:2)];
                    c4=c4+1;
                elseif ~any(S{5,1}(:,1)==q)
                    S{5,1}(c5,1:3)=[q C(i,1:2)];
                    c5=c5+1;
                end
            end
        end
        q=q+1;
    end  
end
% Updated unitvectors
clear uvec
b=length(B);
for i=1:b
    % extracting triad coordinates [x1 y1 z1; x2 y2 z2]
    pos=[N(N==B(i,1),2:4); N(N==B(i,2),2:4)];   
    dx=abs(pos(2,1)-pos(1,1));                  % positive length along axes
    dy=abs(pos(2,2)-pos(1,2));
    dz=abs(pos(2,3)-pos(1,3));
    len=sqrt(dx^2+dy^2+dz^2);                   % length of beam i
    % matrix containing unit vector information [beam# length unitvector]
    uvec(i,1:5)=[i len dx/len dy/len dz/len];   
end
% Defining masterbeams for symmetry from an eigth of the jacket
nl=1;
for i=1:b
    % if one of the beam nodes are on the left half of the master surface
    if (N(N==B(i,1),2)<=(k/2) && N(N==B(i,1),3)==0 && N(N==B(i,2),3)==0) || (N(N==B(i,2),2)<=(k/2) &&  N(N==B(i,1),3)==0 && N(N==B(i,2),3)==0)
        % if NOT one node is on the edge of the master surface and one
        % outside
        if ~((N(N==B(i,1),2)==(k/2) && N(N==B(i,2),2)>(k/2)) || ((N(N==B(i,2),2)==(k/2) && N(N==B(i,1),2)>(k/2))))
            if ~any(S{5,1}(:,1)==i)
                S{6,1}(nl,1:3)=[i B(i,1:2)]; % Masterbeams
                nl=nl+1;
            end
        end
    end
end
% Identifying symmetrical beams by checking 2D position
for s=1:4                                   % for all surfaces
    for i=1:length(S{s,1})                  % for beams on suface s
        if ~any(S{6,1}(:,1)==S{s,1}(i,1))   % if the beam i is not a masterbeam
            Snode(1)=S{s,1}(i,2);           % extracting nodes of slavebeam i        
            Snode(2)=S{s,1}(i,3);
            % Extracting 2D position of nodes for slavebeam i
            STwoDpos(1,1:2)=TwoDpos{s,1}(TwoDpos{s,1}==Snode(1),2:3);   
            STwoDpos(2,1:2)=TwoDpos{s,1}(TwoDpos{s,1}==Snode(2),2:3);          
            % for nodes on other half than mastersurface -> convert 2D
            % position to respective node on left half
            if n > 2
                if (STwoDpos(1,1)>=ceil(n/2) && STwoDpos(2,1)>ceil(n/2)) || (STwoDpos(1,1)>ceil(n/2) && STwoDpos(2,1)>=ceil(n/2))
                    for node=1:2
                        if STwoDpos(node,1)==n
                            STwoDpos(node,1)=1;
                        elseif STwoDpos(node,1)==ceil(n/2)
                            % do nothing
                        else
                            STwoDpos(node,1)=2*(n-STwoDpos(node,1));
                        end
                    end
                end
            end
            for j=1:length(S{6,1}) % for all master beams               
                S{j,2}(1,1)=S{6,1}(j,1);
                % Extracting 2Dpos of masterbeam j
                Mnode(1)=S{6,1}(j,2);
                Mnode(2)=S{6,1}(j,3);
                MTwoDpos(1,1:2)=TwoDpos{1,1}(TwoDpos{1,1}==Mnode(1),2:3);
                MTwoDpos(2,1:2)=TwoDpos{1,1}(TwoDpos{1,1}==Mnode(2),2:3);
                % If master and slavebeam have the same 2D position
                if (all(STwoDpos(1,1:2)==MTwoDpos(1,1:2)) && all(STwoDpos(2,1:2)==MTwoDpos(2,1:2))) || (all(STwoDpos(2,1:2)==MTwoDpos(1,1:2)) && all(STwoDpos(1,1:2)==MTwoDpos(2,1:2)))
                    % if beam i is not already added
                    if ~any(S{j,2}==S{s,1}(i,1))
                        % check that they are of equal length
                        if uvec(S{6,1}(j,1),2)==uvec(S{s,1}(i,1),2) 
                            % Add slavebeam to S{j,2}
                            S{j,2}(1,end+1)=S{s,1}(i,1);
                        end
                    end
                end
            end
        end
    end
end
% Beams crossing the vertical middle need special treatment:
del_cell=0; % for removing cells of concatenated salvebeams
% for all sets of symmetrical beams defined so far
for j=1:length(S{6,1})
    % if the set of slavebeams has not been concatenated earlier
    if ~isempty(S{j,2})
        % extracting a beam (cb(1)=current beam 1)
        cb(1)=S{j,2}(1,1);
        cn(1,1:2)=B(cb(1),1:2);
        % extracting 2Dpos of current beam
        cbTwoDpos{1}(1,1:2)=TwoDpos{1,1}(TwoDpos{1,1}==cn(1,1),2:3);
        cbTwoDpos{1}(2,1:2)=TwoDpos{1,1}(TwoDpos{1,1}==cn(1,2),2:3);
        if  mod(n,2)==0 % for even nodal width
            limit=n/2+0.5;
        else            % for odd nodal width
            limit=ceil(n/2);
        end
        % if beam is crossing the vertical middle line of the face
        if (cbTwoDpos{1}(1,1)>limit && cbTwoDpos{1}(2,1)<limit || (cbTwoDpos{1}(2,1)>limit && cbTwoDpos{1}(1,1)<limit))         
            maxh(1)=max(N(N==cn(1,1),4),N(N==cn(1,2),4)); % max height of beam
            minh(1)=min(N(N==cn(1,1),4),N(N==cn(1,2),4)); % min height of beam
            % for all sets of symmetrical beams
            for i=1:length(S{6,1})
                % if i is not same beam as j && set has not been extracted
                if i~=j && ~isempty(S{i,2})
                cb(2)=S{i,2}(1,1); % extracting current beam 2 
                cn(2,1:2)=B(cb(2),1:2);
                % extracting 2Dpos of current beam 2 
                cbTwoDpos{2}(1,1:2)=TwoDpos{1,1}(TwoDpos{1,1}==cn(2,1),2:3);
                cbTwoDpos{2}(2,1:2)=TwoDpos{1,1}(TwoDpos{1,1}==cn(2,2),2:3);
                maxh(2)=max(N(N==cn(2,1),4),N(N==cn(2,2),4));
                minh(2)=min(N(N==cn(2,1),4),N(N==cn(2,2),4));
                    % if current beam 1 and 2 has the same unit vector and
                    % heigth specifications -> reflect node number of
                    % current beam 2 over the vertical middle line
                    if (uvec(cb(1),2)==uvec(cb(2),2)) && (maxh(1)==maxh(2)) && (minh(1)==minh(2))
                        for node=1:2
                            if cbTwoDpos{2}(node,1)==1  % if node 1 horizontally
                                cbTwoDpos{2}(node,1)=n; % reflect to node n horizontally
                            elseif cbTwoDpos{2}(node,1)==n % etc...
                                cbTwoDpos{2}(node,1)=1;
                            elseif cbTwoDpos{2}(node,1)>ceil(n/2)
                                cbTwoDpos{2}(node,1)=2*(n-cbTwoDpos{2}(1,1));
                            elseif cbTwoDpos{2}(node,1)<ceil(n/2)
                                cbTwoDpos{2}(node,1)=n+1-cbTwoDpos{2}(node,1);
                            end
                        end
                        % if 2Dpos of the two currentbeams coincide one way or the other
                        if (all(cbTwoDpos{2}(1,1:2)==cbTwoDpos{1}(1,1:2)) && all(cbTwoDpos{2}(2,1:2)==cbTwoDpos{1}(2,1:2))) || (all(cbTwoDpos{2}(2,1:2)==cbTwoDpos{1}(1,1:2)) && all(cbTwoDpos{2}(1,1:2)==cbTwoDpos{1}(2,1:2)))
                            % concatenate symmetrical beam set
                            S{j,2}=[S{j,2} S{i,2}];
                            % erase the copied set from its initial cell
                            S{i,2}=[];
                            del_cell(1,end+1)=i;
                        end
                    end
                end
            end
        end
    end
end
% Tiding up S which is a function output
S{end+1,2}(1,1:length(S{5,1}))=S{5,1}(:,1);
S(del_cell(1,2:end),:)=[];
S(:,1)=[];
% each cell of S now contatins beams that will get equal design parameters

%% Adaptions for OC4 transition piece 
tcn=zeros(4,2);
for i=1:length(N) % identifying top corner nodes
    % z coordinate has to be found using tolerance of 0.01 (numerical error)
    if all(N(i,2:3)==[0 0]) && abs(N(i,4)-k*lvls) < 0.01
            tcn(1,1)=N(i,1);
    elseif all(N(i,2:3)==[0 k]) && abs(N(i,4)-k*lvls) < 0.01
            tcn(2,1)=N(i,1);
    elseif all(N(i,2:3)==[k k]) && abs(N(i,4)-k*lvls) < 0.01
            tcn(3,1)=N(i,1);
    elseif all(N(i,2:3)==[k 0]) && abs(N(i,4)-k*lvls) < 0.01
            tcn(4,1)=N(i,1);
    end
end
% moving entire cube to correct position
N(:,2:4)=N(:,2:4)+repmat([-4.016 -4.016 7.619-(lvls-1)*(n-1)*a],m,1);
% defining trasition piece nodes and pariing up with respective jacket nodes
N(end+1,:)=[701 -4 -4 16.15]; % node, plate level 1
tcn(1,2)=N(end,1);            % pairing top corner node with respective plate node
N(end+1,:)=[705 -4 -4 20.15]; % node, plate level 2
N(end+1,:)=[702 -4 4 16.15];
tcn(2,2)=N(end,1);
N(end+1,:)=[706 -4 4 20.15];
N(end+1,:)=[703 4 4 16.15]; 
tcn(3,2)=N(end,1);
N(end+1,:)=[707 4 4 20.15]; 
N(end+1,:)=[704 4 -4 16.15];
tcn(4,2)=N(end,1);
N(end+1,:)=[708 4 -4 20.15];
% Updating beam defenitions
m = length(N);  % number of nodes
for i=m-7:2:m-1 % adding vertical members through transition plate
    B(end+1,:)=[N(i,1) N(i+1,1)];
end
B = [B; tcn];   % connecting cube to vertical t.p. beams

%% Writing fedem .FTL file
b = length(B);  % number of combinations/beams
B(:,1:3) = [zeros(b,1) B(:,1:2)];
ftlname=sprintf('Jacket_%dlvls.ftl',lvls);
fID = fopen(ftlname,'w');
fprintf(fID,'FTLVERSION{4 ASCII}\n');
fprintf(fID,'# Node coordinates\n');
fprintf(fID,'# NODE{id state x y z}\n\n');
for i = 1:m % writing node defenitions
    fprintf(fID,'NODE{%d 0 %d %d %d}\n',N(i,1),N(i,2),N(i,3),N(i,4));
end
fprintf(fID,'\n# Member definitions\n');
fprintf(fID,'# BEAM2{id n1 n2 {PMAT pid}{PBEAMSECTION gid}{PORIENT oid}\n\n');
h=1;
for i = 10001:b+10000 % beam id offset: 10 000
    fprintf(fID,'BEAM2{%d %d %d {PMAT 1} {PBEAMSECTION %d}}\n',i,B(i-10000,2),B(i-10000,3),i);
    B(h,1)=i;   % Adding beam number to first column of B
    h=h+1;
end
fprintf(fID,'\n# Member properties\n');
fprintf(fID,'# PBEAMSECTION{gid a iyy izz ixx ky kz cx cz}\n\n') ;
for i = 10001:b+10000 % writing cross section parameters (dummy), offset 10000
    fprintf(fID,'PBEAMSECTION{%d %d %d %d %d %d %d %d %d}\n',i,0.4398,0.1083,0.1083,0.2166,2,2,0,0);
end
fprintf(fID,'\n# Material properties\n');
fprintf(fID,'# PMAT{pid e g ny rho }\n\n');
fprintf(fID,'PMAT{1 2.10e+11 8.08e+10 0.3 7850 {NAME "Steel"}}\n\n');
fclose(fID);
end % end of function